# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/NeuralActivityCubic',
                'doc_host': 'https://indoc-research.github.io',
                'git_url': 'https://github.com/Indoc-Research/NeuralActivityCubic',
                'lib_path': 'neuralactivitycubic'},
  'syms': { 'neuralactivitycubic.analysis': { 'neuralactivitycubic.analysis.AnalysisROI': ( 'analysis.html#analysisroi',
                                                                                            'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI.__init__': ( 'analysis.html#analysisroi.__init__',
                                                                                                     'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI._add_information_about_neighboring_intersections_to_peaks': ( 'analysis.html#analysisroi._add_information_about_neighboring_intersections_to_peaks',
                                                                                                                                                      'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI._classify_area_under_curve_types': ( 'analysis.html#analysisroi._classify_area_under_curve_types',
                                                                                                                             'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI._convert_polygon_to_mask': ( 'analysis.html#analysisroi._convert_polygon_to_mask',
                                                                                                                     'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI._get_unique_frame_idxs_of_intersections_between_signal_and_baseline': ( 'analysis.html#analysisroi._get_unique_frame_idxs_of_intersections_between_signal_and_baseline',
                                                                                                                                                                'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI._improve_intersection_frame_idx_estimation_by_interpolation': ( 'analysis.html#analysisroi._improve_intersection_frame_idx_estimation_by_interpolation',
                                                                                                                                                        'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI.compute_amplitude_and_delta_f_over_f': ( 'analysis.html#analysisroi.compute_amplitude_and_delta_f_over_f',
                                                                                                                                 'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI.compute_area_under_curve': ( 'analysis.html#analysisroi.compute_area_under_curve',
                                                                                                                     'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI.compute_mean_intensity_timeseries': ( 'analysis.html#analysisroi.compute_mean_intensity_timeseries',
                                                                                                                              'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI.compute_variance_area': ( 'analysis.html#analysisroi.compute_variance_area',
                                                                                                                  'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI.detect_peaks': ( 'analysis.html#analysisroi.detect_peaks',
                                                                                                         'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.AnalysisROI.estimate_baseline': ( 'analysis.html#analysisroi.estimate_baseline',
                                                                                                              'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.BaselineEstimatorFactory': ( 'analysis.html#baselineestimatorfactory',
                                                                                                         'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.BaselineEstimatorFactory.get_baseline_estimation_callable': ( 'analysis.html#baselineestimatorfactory.get_baseline_estimation_callable',
                                                                                                                                          'neuralactivitycubic/analysis.py'),
                                              'neuralactivitycubic.analysis.BaselineEstimatorFactory.supported_baseline_estimation_methods': ( 'analysis.html#baselineestimatorfactory.supported_baseline_estimation_methods',
                                                                                                                                               'neuralactivitycubic/analysis.py')},
            'neuralactivitycubic.api': {'neuralactivitycubic.api.run_analysis': ('api.html#run_analysis', 'neuralactivitycubic/api.py')},
            'neuralactivitycubic.controller': { 'neuralactivitycubic.controller.App': ( 'controller.html#app',
                                                                                        'neuralactivitycubic/controller.py'),
                                                'neuralactivitycubic.controller.App.__init__': ( 'controller.html#app.__init__',
                                                                                                 'neuralactivitycubic/controller.py'),
                                                'neuralactivitycubic.controller.App._bind_buttons_of_view_to_functions_of_model': ( 'controller.html#app._bind_buttons_of_view_to_functions_of_model',
                                                                                                                                    'neuralactivitycubic/controller.py'),
                                                'neuralactivitycubic.controller.App._display_preview_of_representative_job': ( 'controller.html#app._display_preview_of_representative_job',
                                                                                                                               'neuralactivitycubic/controller.py'),
                                                'neuralactivitycubic.controller.App._load_data_button_clicked': ( 'controller.html#app._load_data_button_clicked',
                                                                                                                  'neuralactivitycubic/controller.py'),
                                                'neuralactivitycubic.controller.App._preview_window_size_button_clicked': ( 'controller.html#app._preview_window_size_button_clicked',
                                                                                                                            'neuralactivitycubic/controller.py'),
                                                'neuralactivitycubic.controller.App._run_button_clicked': ( 'controller.html#app._run_button_clicked',
                                                                                                            'neuralactivitycubic/controller.py'),
                                                'neuralactivitycubic.controller.App._setup_interaction_between_model_and_view': ( 'controller.html#app._setup_interaction_between_model_and_view',
                                                                                                                                  'neuralactivitycubic/controller.py'),
                                                'neuralactivitycubic.controller.App.launch': ( 'controller.html#app.launch',
                                                                                               'neuralactivitycubic/controller.py'),
                                                'neuralactivitycubic.controller.open_gui': ( 'controller.html#open_gui',
                                                                                             'neuralactivitycubic/controller.py')},
            'neuralactivitycubic.datamodels': { 'neuralactivitycubic.datamodels.BaseDataClass': ( 'datamodels.html#basedataclass',
                                                                                                  'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.BaseDataClass.display_all_attributes': ( 'datamodels.html#basedataclass.display_all_attributes',
                                                                                                                         'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.BaseDataClass.from_dict': ( 'datamodels.html#basedataclass.from_dict',
                                                                                                            'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.BaseDataClass.to_dict': ( 'datamodels.html#basedataclass.to_dict',
                                                                                                          'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.BaseDataClass.validate': ( 'datamodels.html#basedataclass.validate',
                                                                                                           'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.BaselineEstimationMethod': ( 'datamodels.html#baselineestimationmethod',
                                                                                                             'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.BaselineEstimationMethod.__str__': ( 'datamodels.html#baselineestimationmethod.__str__',
                                                                                                                     'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.Config': ( 'datamodels.html#config',
                                                                                           'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.Config.__post_init__': ( 'datamodels.html#config.__post_init__',
                                                                                                         'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.Config.to_json': ( 'datamodels.html#config.to_json',
                                                                                                   'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.Peak': ( 'datamodels.html#peak',
                                                                                         'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.ROIMode': ( 'datamodels.html#roimode',
                                                                                            'neuralactivitycubic/datamodels.py'),
                                                'neuralactivitycubic.datamodels.ROIMode.__str__': ( 'datamodels.html#roimode.__str__',
                                                                                                    'neuralactivitycubic/datamodels.py')},
            'neuralactivitycubic.input': { 'neuralactivitycubic.input.AVILoader': ('input.html#aviloader', 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.AVILoader._get_all_frames': ( 'input.html#aviloader._get_all_frames',
                                                                                                    'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.Data': ('input.html#data', 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.Data.__init__': ( 'input.html#data.__init__',
                                                                                        'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.Data._parse_loaded_data': ( 'input.html#data._parse_loaded_data',
                                                                                                  'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.DataLoader': ( 'input.html#dataloader',
                                                                                     'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.DataLoader.__init__': ( 'input.html#dataloader.__init__',
                                                                                              'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.DataLoader.load_and_parse_file_content': ( 'input.html#dataloader.load_and_parse_file_content',
                                                                                                                 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.DataLoaderFactory': ( 'input.html#dataloaderfactory',
                                                                                            'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.DataLoaderFactory._assert_validity_of_filepath': ( 'input.html#dataloaderfactory._assert_validity_of_filepath',
                                                                                                                         'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.DataLoaderFactory._get_loader_for_file_extension': ( 'input.html#dataloaderfactory._get_loader_for_file_extension',
                                                                                                                           'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.DataLoaderFactory.all_supported_extensions': ( 'input.html#dataloaderfactory.all_supported_extensions',
                                                                                                                     'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.DataLoaderFactory.get_loader': ( 'input.html#dataloaderfactory.get_loader',
                                                                                                       'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.DataLoaderFactory.supported_extensions_per_data_loader': ( 'input.html#dataloaderfactory.supported_extensions_per_data_loader',
                                                                                                                                 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.FocusAreaPathRestrictions': ( 'input.html#focusareapathrestrictions',
                                                                                                    'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.FocusAreaPathRestrictions.supported_dir_names': ( 'input.html#focusareapathrestrictions.supported_dir_names',
                                                                                                                        'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.GridWrapperROILoader': ( 'input.html#gridwrapperroiloader',
                                                                                               'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.GridWrapperROILoader._add_to_configs_and_create_as_attribute': ( 'input.html#gridwrapperroiloader._add_to_configs_and_create_as_attribute',
                                                                                                                                       'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.GridWrapperROILoader._get_boundary_row_col_coords_single_square': ( 'input.html#gridwrapperroiloader._get_boundary_row_col_coords_single_square',
                                                                                                                                          'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.GridWrapperROILoader._get_cropping_indices_to_adjust_for_window_size': ( 'input.html#gridwrapperroiloader._get_cropping_indices_to_adjust_for_window_size',
                                                                                                                                               'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.GridWrapperROILoader._get_row_col_idxs_of_grid': ( 'input.html#gridwrapperroiloader._get_row_col_idxs_of_grid',
                                                                                                                         'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.GridWrapperROILoader._get_row_col_labels_for_rois_in_grid': ( 'input.html#gridwrapperroiloader._get_row_col_labels_for_rois_in_grid',
                                                                                                                                    'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.GridWrapperROILoader.load_and_parse_file_content': ( 'input.html#gridwrapperroiloader.load_and_parse_file_content',
                                                                                                                           'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.GridWrapperROILoader.set_configs_for_grid_creation': ( 'input.html#gridwrapperroiloader.set_configs_for_grid_creation',
                                                                                                                             'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ImageJROILoader': ( 'input.html#imagejroiloader',
                                                                                          'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ImageJROILoader._extract_boundary_row_col_coords_from_roi_set': ( 'input.html#imagejroiloader._extract_boundary_row_col_coords_from_roi_set',
                                                                                                                                        'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ImageJROILoader._extract_boundary_row_col_coords_from_single_roi': ( 'input.html#imagejroiloader._extract_boundary_row_col_coords_from_single_roi',
                                                                                                                                           'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ImageJROILoader._get_boundary_row_col_coords_for_all_rois_in_source_data': ( 'input.html#imagejroiloader._get_boundary_row_col_coords_for_all_rois_in_source_data',
                                                                                                                                                   'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.NWBROILoader': ( 'input.html#nwbroiloader',
                                                                                       'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.NWBROILoader._get_boundary_row_col_coords_for_all_rois_in_source_data': ( 'input.html#nwbroiloader._get_boundary_row_col_coords_for_all_rois_in_source_data',
                                                                                                                                                'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.NWBRecordingLoader': ( 'input.html#nwbrecordingloader',
                                                                                             'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.NWBRecordingLoader._get_all_frames': ( 'input.html#nwbrecordingloader._get_all_frames',
                                                                                                             'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROI': ('input.html#roi', 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROI._convert_to_valid_polygon': ( 'input.html#roi._convert_to_valid_polygon',
                                                                                                        'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROI._parse_loaded_data': ( 'input.html#roi._parse_loaded_data',
                                                                                                 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROI.add_label_id': ( 'input.html#roi.add_label_id',
                                                                                           'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROI.create_nwb_compliant_pixel_mask': ( 'input.html#roi.create_nwb_compliant_pixel_mask',
                                                                                                              'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROILoader': ('input.html#roiloader', 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROILoader._add_first_boundary_point_also_add_end_to_close_roi': ( 'input.html#roiloader._add_first_boundary_point_also_add_end_to_close_roi',
                                                                                                                                        'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROILoader._get_boundary_row_col_coords_for_all_rois_in_source_data': ( 'input.html#roiloader._get_boundary_row_col_coords_for_all_rois_in_source_data',
                                                                                                                                             'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROILoader.load_and_parse_file_content': ( 'input.html#roiloader.load_and_parse_file_content',
                                                                                                                'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROILoaderFactory': ( 'input.html#roiloaderfactory',
                                                                                           'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.ROILoaderFactory.supported_extensions_per_data_loader': ( 'input.html#roiloaderfactory.supported_extensions_per_data_loader',
                                                                                                                                'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecLoaderROILoaderCombinator': ( 'input.html#recloaderroiloadercombinator',
                                                                                                       'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecLoaderROILoaderCombinator.__init__': ( 'input.html#recloaderroiloadercombinator.__init__',
                                                                                                                'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecLoaderROILoaderCombinator._get_all_roi_loaders': ( 'input.html#recloaderroiloadercombinator._get_all_roi_loaders',
                                                                                                                            'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecLoaderROILoaderCombinator._get_the_recording_loader': ( 'input.html#recloaderroiloadercombinator._get_the_recording_loader',
                                                                                                                                 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecLoaderROILoaderCombinator.get_all_recording_and_roi_loader_combos': ( 'input.html#recloaderroiloadercombinator.get_all_recording_and_roi_loader_combos',
                                                                                                                                               'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.Recording': ('input.html#recording', 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.Recording._clip_image_at_bit_values': ( 'input.html#recording._clip_image_at_bit_values',
                                                                                                              'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.Recording._compute_contrast_and_brightness_adjustment_factors': ( 'input.html#recording._compute_contrast_and_brightness_adjustment_factors',
                                                                                                                                        'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.Recording._create_brightness_and_contrast_enhanced_preview': ( 'input.html#recording._create_brightness_and_contrast_enhanced_preview',
                                                                                                                                     'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.Recording._estimate_bit_depth': ( 'input.html#recording._estimate_bit_depth',
                                                                                                        'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.Recording._parse_loaded_data': ( 'input.html#recording._parse_loaded_data',
                                                                                                       'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecordingLoader': ( 'input.html#recordingloader',
                                                                                          'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecordingLoader._check_if_color_channels_are_redundant': ( 'input.html#recordingloader._check_if_color_channels_are_redundant',
                                                                                                                                 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecordingLoader._check_if_multiple_color_channels_exist': ( 'input.html#recordingloader._check_if_multiple_color_channels_exist',
                                                                                                                                  'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecordingLoader._convert_to_grayscale': ( 'input.html#recordingloader._convert_to_grayscale',
                                                                                                                'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecordingLoader._ensure_array_is_4d': ( 'input.html#recordingloader._ensure_array_is_4d',
                                                                                                              'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecordingLoader._get_all_frames': ( 'input.html#recordingloader._get_all_frames',
                                                                                                          'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecordingLoader._load_all_frames': ( 'input.html#recordingloader._load_all_frames',
                                                                                                           'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecordingLoader.load_and_parse_file_content': ( 'input.html#recordingloader.load_and_parse_file_content',
                                                                                                                      'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecordingLoaderFactory': ( 'input.html#recordingloaderfactory',
                                                                                                 'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.RecordingLoaderFactory.supported_extensions_per_data_loader': ( 'input.html#recordingloaderfactory.supported_extensions_per_data_loader',
                                                                                                                                      'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.TIFFLoader': ( 'input.html#tiffloader',
                                                                                     'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.TIFFLoader._get_all_frames': ( 'input.html#tiffloader._get_all_frames',
                                                                                                     'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.get_filepaths_with_supported_extension_in_dirpath': ( 'input.html#get_filepaths_with_supported_extension_in_dirpath',
                                                                                                                            'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.test_successful_recording_loading': ( 'input.html#test_successful_recording_loading',
                                                                                                            'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.test_successful_roi_loading': ( 'input.html#test_successful_roi_loading',
                                                                                                      'neuralactivitycubic/input.py'),
                                           'neuralactivitycubic.input.test_unsupported_file_extension': ( 'input.html#test_unsupported_file_extension',
                                                                                                          'neuralactivitycubic/input.py')},
            'neuralactivitycubic.model': { 'neuralactivitycubic.model.Logger': ('model.html#logger', 'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Logger.__init__': ( 'model.html#logger.__init__',
                                                                                          'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Logger.add_new_log': ( 'model.html#logger.add_new_log',
                                                                                             'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Logger.clear_logs': ( 'model.html#logger.clear_logs',
                                                                                            'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Logger.get_logs': ( 'model.html#logger.get_logs',
                                                                                          'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Logger.save_current_logs': ( 'model.html#logger.save_current_logs',
                                                                                                   'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model': ('model.html#model', 'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model.__init__': ( 'model.html#model.__init__',
                                                                                         'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model._create_analysis_jobs_for_single_rec': ( 'model.html#model._create_analysis_jobs_for_single_rec',
                                                                                                                     'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model._create_single_analysis_job': ( 'model.html#model._create_single_analysis_job',
                                                                                                            'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model._display_configs': ( 'model.html#model._display_configs',
                                                                                                 'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model._ensure_data_from_previous_jobs_was_removed': ( 'model.html#model._ensure_data_from_previous_jobs_was_removed',
                                                                                                                            'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model._get_all_roi_loaders': ( 'model.html#model._get_all_roi_loaders',
                                                                                                     'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model._get_all_subdir_paths_with_rec_file': ( 'model.html#model._get_all_subdir_paths_with_rec_file',
                                                                                                                    'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model._get_focus_area_dir_path': ( 'model.html#model._get_focus_area_dir_path',
                                                                                                         'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model._get_recording_loader': ( 'model.html#model._get_recording_loader',
                                                                                                      'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model._save_user_settings_as_json': ( 'model.html#model._save_user_settings_as_json',
                                                                                                            'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model.add_info_to_logs': ( 'model.html#model.add_info_to_logs',
                                                                                                 'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model.create_analysis_jobs': ( 'model.html#model.create_analysis_jobs',
                                                                                                     'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model.preview_window_size': ( 'model.html#model.preview_window_size',
                                                                                                    'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model.run_analysis': ( 'model.html#model.run_analysis',
                                                                                             'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model.setup_connection_to_display_results': ( 'model.html#model.setup_connection_to_display_results',
                                                                                                                    'neuralactivitycubic/model.py'),
                                           'neuralactivitycubic.model.Model.setup_connection_to_update_infos_in_view': ( 'model.html#model.setup_connection_to_update_infos_in_view',
                                                                                                                         'neuralactivitycubic/model.py')},
            'neuralactivitycubic.processing': { 'neuralactivitycubic.processing.AnalysisJob': ( 'processing.html#analysisjob',
                                                                                                'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob.__init__': ( 'processing.html#analysisjob.__init__',
                                                                                                         'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._add_na3_results': ( 'processing.html#analysisjob._add_na3_results',
                                                                                                                 'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._add_source_data': ( 'processing.html#analysisjob._add_source_data',
                                                                                                                 'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._create_all_analysis_rois': ( 'processing.html#analysisjob._create_all_analysis_rois',
                                                                                                                          'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._create_analysis_rois': ( 'processing.html#analysisjob._create_analysis_rois',
                                                                                                                      'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._create_and_add_label_ids_to_all_rois_from_file': ( 'processing.html#analysisjob._create_and_add_label_ids_to_all_rois_from_file',
                                                                                                                                                'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._create_and_save_csv_result_files': ( 'processing.html#analysisjob._create_and_save_csv_result_files',
                                                                                                                                  'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._create_and_save_individual_traces_pdf_result_file': ( 'processing.html#analysisjob._create_and_save_individual_traces_pdf_result_file',
                                                                                                                                                   'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._create_and_save_single_trace_results_as_csv': ( 'processing.html#analysisjob._create_and_save_single_trace_results_as_csv',
                                                                                                                                             'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._create_results_dir': ( 'processing.html#analysisjob._create_results_dir',
                                                                                                                    'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._create_variance_area_dataframe': ( 'processing.html#analysisjob._create_variance_area_dataframe',
                                                                                                                                'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._export_all_data_to_nwb': ( 'processing.html#analysisjob._export_all_data_to_nwb',
                                                                                                                        'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._filter_rois_by_focus_area': ( 'processing.html#analysisjob._filter_rois_by_focus_area',
                                                                                                                           'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._get_info_for_nwb_units_table_from_analysis_roi': ( 'processing.html#analysisjob._get_info_for_nwb_units_table_from_analysis_roi',
                                                                                                                                                'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._initialize_nwb_file': ( 'processing.html#analysisjob._initialize_nwb_file',
                                                                                                                     'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._initialize_single_trace_df': ( 'processing.html#analysisjob._initialize_single_trace_df',
                                                                                                                            'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._load_data_from_all_roi_loaders': ( 'processing.html#analysisjob._load_data_from_all_roi_loaders',
                                                                                                                                'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._save_single_trace_df': ( 'processing.html#analysisjob._save_single_trace_df',
                                                                                                                      'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob._write_nwb_file': ( 'processing.html#analysisjob._write_nwb_file',
                                                                                                                'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob.create_results': ( 'processing.html#analysisjob.create_results',
                                                                                                               'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob.load_data_into_memory': ( 'processing.html#analysisjob.load_data_into_memory',
                                                                                                                      'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob.preview_window_size': ( 'processing.html#analysisjob.preview_window_size',
                                                                                                                    'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.AnalysisJob.run_analysis': ( 'processing.html#analysisjob.run_analysis',
                                                                                                             'neuralactivitycubic/processing.py'),
                                                'neuralactivitycubic.processing.process_analysis_rois': ( 'processing.html#process_analysis_rois',
                                                                                                          'neuralactivitycubic/processing.py')},
            'neuralactivitycubic.results': { 'neuralactivitycubic.results._get_all_peak_counts': ( 'results.html#_get_all_peak_counts',
                                                                                                   'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._get_fontsize_adjusted_to_slimmest_roi': ( 'results.html#_get_fontsize_adjusted_to_slimmest_roi',
                                                                                                                     'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._get_max_label_id_length': ( 'results.html#_get_max_label_id_length',
                                                                                                       'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._get_max_peak_count': ( 'results.html#_get_max_peak_count',
                                                                                                  'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._get_min_width_among_all_rois': ( 'results.html#_get_min_width_among_all_rois',
                                                                                                            'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._get_text_bounding_box_size_in_data_dimensions': ( 'results.html#_get_text_bounding_box_size_in_data_dimensions',
                                                                                                                             'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._get_total_peak_count': ( 'results.html#_get_total_peak_count',
                                                                                                    'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._iteratively_decrease_fontsize_to_fit_text_in_squares': ( 'results.html#_iteratively_decrease_fontsize_to_fit_text_in_squares',
                                                                                                                                    'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._plot_grid': ( 'results.html#_plot_grid',
                                                                                         'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._plot_peak_count_text_for_each_analysis_roi': ( 'results.html#_plot_peak_count_text_for_each_analysis_roi',
                                                                                                                          'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._plot_text_at_roi_centroid_coordinates': ( 'results.html#_plot_text_at_roi_centroid_coordinates',
                                                                                                                     'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results._use_grid_row_col_labels_as_axis_labels': ( 'results.html#_use_grid_row_col_labels_as_axis_labels',
                                                                                                                      'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results.create_single_roi_amplitude_and_delta_f_over_f_results': ( 'results.html#create_single_roi_amplitude_and_delta_f_over_f_results',
                                                                                                                                     'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results.create_single_roi_auc_results': ( 'results.html#create_single_roi_auc_results',
                                                                                                            'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results.export_peak_results_df_from_analysis_roi': ( 'results.html#export_peak_results_df_from_analysis_roi',
                                                                                                                       'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results.plot_activity_overview': ( 'results.html#plot_activity_overview',
                                                                                                     'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results.plot_intensity_trace_with_identified_peaks_for_individual_roi': ( 'results.html#plot_intensity_trace_with_identified_peaks_for_individual_roi',
                                                                                                                                            'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results.plot_roi_boundaries': ( 'results.html#plot_roi_boundaries',
                                                                                                  'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results.plot_rois_with_label_id_overview': ( 'results.html#plot_rois_with_label_id_overview',
                                                                                                               'neuralactivitycubic/results.py'),
                                             'neuralactivitycubic.results.plot_window_size_preview': ( 'results.html#plot_window_size_preview',
                                                                                                       'neuralactivitycubic/results.py')},
            'neuralactivitycubic.view': { 'neuralactivitycubic.view.AnalysisSettingsPanel': ( 'view.html#analysissettingspanel',
                                                                                              'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.AnalysisSettingsPanel.AnalysisSettings': ( 'view.html#analysissettingspanel.analysissettings',
                                                                                                               'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.AnalysisSettingsPanel.__init__': ( 'view.html#analysissettingspanel.__init__',
                                                                                                       'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.AnalysisSettingsPanel._configure_octaves_changed': ( 'view.html#analysissettingspanel._configure_octaves_changed',
                                                                                                                         'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.AnalysisSettingsPanel._include_variance_config_changed': ( 'view.html#analysissettingspanel._include_variance_config_changed',
                                                                                                                               'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.AnalysisSettingsPanel._limit_analysis_to_interval_changed': ( 'view.html#analysissettingspanel._limit_analysis_to_interval_changed',
                                                                                                                                  'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.AnalysisSettingsPanel.enable_analysis_settings': ( 'view.html#analysissettingspanel.enable_analysis_settings',
                                                                                                                       'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.AnalysisSettingsPanel.widget': ( 'view.html#analysissettingspanel.widget',
                                                                                                     'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.MainScreen': ('view.html#mainscreen', 'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.MainScreen.__init__': ( 'view.html#mainscreen.__init__',
                                                                                            'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.MainScreen.output_screen': ( 'view.html#mainscreen.output_screen',
                                                                                                 'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.MainScreen.show_output_screen': ( 'view.html#mainscreen.show_output_screen',
                                                                                                      'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.MainScreen.show_welcome_screen': ( 'view.html#mainscreen.show_welcome_screen',
                                                                                                       'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.MainScreen.welcome_screen': ( 'view.html#mainscreen.welcome_screen',
                                                                                                  'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.NWBMetadataPanel': ( 'view.html#nwbmetadatapanel',
                                                                                         'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.NWBMetadataPanel.NWBMetadata': ( 'view.html#nwbmetadatapanel.nwbmetadata',
                                                                                                     'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.NWBMetadataPanel.__init__': ( 'view.html#nwbmetadatapanel.__init__',
                                                                                                  'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.NWBMetadataPanel.widget': ( 'view.html#nwbmetadatapanel.widget',
                                                                                                'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.SourceDataPanel': ( 'view.html#sourcedatapanel',
                                                                                        'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.SourceDataPanel.SourceDataSettings': ( 'view.html#sourcedatapanel.sourcedatasettings',
                                                                                                           'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.SourceDataPanel.__init__': ( 'view.html#sourcedatapanel.__init__',
                                                                                                 'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.SourceDataPanel._change_batch_mode_config': ( 'view.html#sourcedatapanel._change_batch_mode_config',
                                                                                                                  'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.SourceDataPanel._change_data_source_path_configs': ( 'view.html#sourcedatapanel._change_data_source_path_configs',
                                                                                                                         'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.SourceDataPanel._change_focus_area_config': ( 'view.html#sourcedatapanel._change_focus_area_config',
                                                                                                                  'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.SourceDataPanel._change_roi_mode_config': ( 'view.html#sourcedatapanel._change_roi_mode_config',
                                                                                                                'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.SourceDataPanel.widget': ( 'view.html#sourcedatapanel.widget',
                                                                                               'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.UserInfoPanel': ( 'view.html#userinfopanel',
                                                                                      'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.UserInfoPanel.__init__': ( 'view.html#userinfopanel.__init__',
                                                                                               'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.UserInfoPanel.add_new_logs': ( 'view.html#userinfopanel.add_new_logs',
                                                                                                   'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.UserInfoPanel.update_progress_bar': ( 'view.html#userinfopanel.update_progress_bar',
                                                                                                          'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.UserInfoPanel.widget': ( 'view.html#userinfopanel.widget',
                                                                                             'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface': ( 'view.html#widgetsinterface',
                                                                                         'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface.__init__': ( 'view.html#widgetsinterface.__init__',
                                                                                                  'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface._data_source_path_chosen': ( 'view.html#widgetsinterface._data_source_path_chosen',
                                                                                                                  'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface._enable_window_size_widgets': ( 'view.html#widgetsinterface._enable_window_size_widgets',
                                                                                                                     'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface.adjust_widgets_to_loaded_data': ( 'view.html#widgetsinterface.adjust_widgets_to_loaded_data',
                                                                                                                       'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface.data_source_path': ( 'view.html#widgetsinterface.data_source_path',
                                                                                                          'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface.enable_analysis': ( 'view.html#widgetsinterface.enable_analysis',
                                                                                                         'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface.export_nwb_metadata': ( 'view.html#widgetsinterface.export_nwb_metadata',
                                                                                                             'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface.export_user_settings': ( 'view.html#widgetsinterface.export_user_settings',
                                                                                                              'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface.grid_size': ( 'view.html#widgetsinterface.grid_size',
                                                                                                   'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.WidgetsInterface.update_infos': ( 'view.html#widgetsinterface.update_infos',
                                                                                                      'neuralactivitycubic/view.py'),
                                          'neuralactivitycubic.view.change_widget_state': ( 'view.html#change_widget_state',
                                                                                            'neuralactivitycubic/view.py')}}}
