# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/08_api.ipynb.

# %% auto 0
__all__ = ['run_analysis']

# %% ../nbs/08_api.ipynb 2
from .datamodels import Config
from .model import Model

def run_analysis(config: Config | str) -> None:
    """
    Run analysis.

    Args:
        config (Config | str): Configuration for analysis or data source path to run analysis with default settings.
    """
    model = Model(config)
    model.create_analysis_jobs()
    model.run_analysis()
